//
//  IAPHelper.swift
//  selecttv
//
//  Created by Akash Patel on 01/06/20.
//  Copyright © 2020 FreeCast Inc. All rights reserved.
//

import Foundation
import StoreKit
//import SwiftyJSON

class IAPHelper : NSObject {
    
    static let shared = IAPHelper()
    
    var initialisedStoreKit = false
    
    fileprivate func initializeStoreKit() {
        SwiftyStoreKit.completeTransactions(atomically: true) { purchases in
            for purchase in purchases {
                switch purchase.transaction.transactionState {
                case .purchased, .restored:
                    if purchase.needsFinishTransaction {
                        // Deliver content from server, then:
                        SwiftyStoreKit.finishTransaction(purchase.transaction)
                    }
                    // Unlock content
                case .failed, .purchasing, .deferred:
                    break // do nothing
                }
            }
        }
    }
    
    func startInAppPurchaseWith(productIdentifier : String, iosAppSecret: String, _ completion : @escaping (_ errorMessage : String?, _ transaction : PurchaseDetails?, _ transactionID : String?)->Void) {
        
        if !initialisedStoreKit {
            self.initializeStoreKit()
        }
        
        SwiftyStoreKit.purchaseProduct(productIdentifier, atomically: true) { result in
            
            var errorMessage : String?
            switch result {
            case .success(let purchase):
                print("Purchase Success: \(purchase.productId)")
                completion(nil, purchase, purchase.transaction.transactionIdentifier)
                break
            case .error(let error):
                switch error.code {
                case .unknown: print("Unknown error. Please contact support")
                    errorMessage = "Unknown error. Please contact support"
                case .clientInvalid: print("Not allowed to make the payment")
                    errorMessage = "Not allowed to make the payment"
                case .paymentCancelled:
                    errorMessage = "Payment cancelled by user"
                    break
                case .paymentInvalid: print("The purchase identifier was invalid")
                    errorMessage = "The purchase identifier was invalid"
                case .paymentNotAllowed: print("The device is not allowed to make the payment")
                    errorMessage = "The device is not allowed to make the payment"
                case .storeProductNotAvailable: print("The product is not available in the current storefront")
                    errorMessage = "The product is not available in the current storefront"
                case .cloudServicePermissionDenied: print("Access to cloud service information is not allowed")
                    errorMessage = "Access to cloud service information is not allowed"
                case .cloudServiceNetworkConnectionFailed: print("Could not connect to the network")
                    errorMessage = "Could not connect to the network"
                case .cloudServiceRevoked: print("User has revoked permission to use this cloud service")
                    errorMessage = "User has revoked permission to use this cloud service"
                default: print((error as NSError).localizedDescription)
                    errorMessage = (error as NSError).localizedDescription
                }
                
                completion(errorMessage, nil, nil)
                break
            }
        }
    }
    
    func getProductInfo(productIdentifier : String, iosAppSecret: String, _ completion : @escaping (_ productIdentifier : String?, _ transaction : SKProduct?, _ priceWithCurrency: String?)->Void) {
        SwiftyStoreKit.retrieveProductsInfo(Set.init([productIdentifier])) { result in
            if let product = result.retrievedProducts.first {
                let priceString = product.localizedPrice!
                print("Product: \(product.localizedDescription), price: \(priceString)")
                completion(product.productIdentifier, product, priceString)
            }
            else if let invalidProductId = result.invalidProductIDs.first {
                print("Invalid product identifier: \(invalidProductId)")
                completion(nil, nil, nil)
            }
            else {
                print("Error: \(result.error)")
                completion(nil, nil, nil)
            }
        }
        
    }
    
    
    
    func restorePurchase(_ completion : @escaping (_ errorMessage : String?, _ lastPurchase : Purchase?)->Void) {
        SwiftyStoreKit.restorePurchases(atomically: true) { results in
            if results.restoreFailedPurchases.count > 0 {
                print("Restore Failed: \(results.restoreFailedPurchases)")
                completion("Restore Failed: \(results.restoreFailedPurchases)", nil)
            }
            else if results.restoredPurchases.count > 0 {
                print("Restore Success: \(results.restoredPurchases)")
                let arrangedTransactions = results.restoredPurchases.sorted {
                    guard let date1 = $0.transaction.transactionDate else {return false}
                    guard let date2 = $1.transaction.transactionDate else {return false}
                    return date1 < date2
                }
                print(arrangedTransactions)
                
                completion(nil, arrangedTransactions.last)
                
            }
            else {
                print("Nothing to Restore")
                completion("Nothing to Restore", nil)
            }
        }
    }
}

