import Flutter
import UIKit

public class SwiftSpectusInAppPurchasePlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "spectus_in_app_purchase", binaryMessenger: registrar.messenger())
        let instance = SwiftSpectusInAppPurchasePlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if (call.method == "getPlatformVersion") {
            result("iOS " + UIDevice.current.systemVersion)
        }
        else if (call.method == "startInAppPurchaseWith") {
            //In-App Purchase Payment
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any], let prodId = myArgs["productID"] as? String, let iosAppSecretVal = myArgs["iosAppSecret"] as? String {
                IAPHelper.shared.startInAppPurchaseWith(productIdentifier : prodId, iosAppSecret: iosAppSecretVal, { (errorMessage, transactionObj, transactionID) in
                    if errorMessage != nil {
                        result("NA")
                    } else {
                        result(transactionID)
                    }
                })
            }
        }
        else if (call.method == "startRestorePurchase") {
            //In-App Restore Purchase
            IAPHelper.shared.restorePurchase({ (errorMessage, lastTransaction) in
                if errorMessage != nil {
                    result("NA")
                } else {

                    let transactionID = lastTransaction?.transaction.transactionIdentifier ?? "NA"
                    var dateOfTransaction = "NA"
                    if let transactionDate = lastTransaction?.transaction.transactionDate {
                        let dateF = DateFormatter.init()
                        dateF.dateFormat = "dd/MM/yyyy HH:mm"
                        dateF.locale = Locale.current
                        dateF.timeZone = TimeZone.current
                        dateOfTransaction = dateF.string(from: transactionDate)
                    }
                    let stringWithTransactionData = "\(transactionID)&^&\(dateOfTransaction)&^&\(lastTransaction?.productId ?? "NA")"
                    result(stringWithTransactionData)
                }
            })
        }
        else if (call.method == "getProductDetailsOf") {
            guard let args = call.arguments else {
                return
            }
            if let myArgs = args as? [String: Any], let prodId = myArgs["productID"] as? String, let iosAppSecretVal = myArgs["iosAppSecret"] as? String {
                IAPHelper.shared.getProductInfo(productIdentifier: prodId, iosAppSecret: iosAppSecretVal) { productIdentifier, productObject, priceString in
                    print("Completion In Swift => \n Product ID => \(productIdentifier)\n Product => \(productObject?.description ?? "")\n=======================\n")
                    
                    if let product = productObject {
                        print("\n=> \(product.productIdentifier)")
                        print("\n=> \(product.price)")
                        print("\n=> \(product.priceLocale)")
                        print("\n=> \(product.priceLocale.currencyCode)")
                        print("\n=> \(product.priceLocale.currencySymbol)")
                        print("\n=> \(getSymbolForCurrencyCode(code: product.priceLocale.identifier))")
                        print("\n=> \(product.localizedTitle)")
                        print("\n=> \(product.localizedDescription)")
                        
                        let formatter = NumberFormatter()
                        formatter.numberStyle = .currency
                        let priceWithFormat = formatter.string(from: product.price) ?? product.price.description
                        let stringWithSeparator = "\(product.productIdentifier)&^&\(priceString)&^&\(product.priceLocale.currencyCode ?? (Locale.current.currencyCode ?? ""))&^&\(product.priceLocale.currencySymbol ?? (Locale.current.currencySymbol ?? ""))&^&\(product.localizedTitle.isEmpty ? "NA" : product.localizedTitle)&^&\(product.localizedDescription.isEmpty ? "NA" : product.localizedDescription)"
                        result(stringWithSeparator)
                    } else {
                        result("NA")
                    }
                }
            }
        }
        else {
            result("Failed to get top controller")
        }
    }
}
